using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO.Ports;
using System.IO;

namespace CAPNO_2_PC
{
    public partial class Form1 : Form
    {
        private delegate void displayDeleg(byte command,Int16 value);
        private delegate void InfoDeleg(string stream);
        private delegate void ErrorDeleg(string stream);
        private delegate void ParameterDeleg(string stream);

        int index = 0;
        bool breath_trigger = false;

        static string[] ErrorMessage = {"Normal Operation",
                                 "Flash Checksum !!!",
                                 "Bench EEprom !!!",
                                 "Program EEprom !!!",
                                 "Bench Data Corrupted !!!",
                                 "EEprom Data Corrupted !!!",
                                 "Line Occluded",
                                 "No Flow !!!",
                                 "Flow Too Low !!!",
                                 "Flow Too High !!!",
                                 "Zero Reference Too High !!!",
                                 "Negative CO2 !!!"};

        static string[] StatusMessage = {"No Info Available",
                                  "Zero Correction Started",
                                  "Zero Correction Ended",
                                  "5% Calibration Started",
                                  "5% Calibration Ended",
                                  "Recalibrate!",
                                  "Flash Checksum ok",
                                  "Bench Calibration Data ok",
                                  "Gas Sampling Started",
                                  "No Filter !",
                                  "Correction Factor Set",
                                  "Gain Factor Set",
                                  "Init Pump PWM",
                                  "K Coefficient Table Built",
                                  "J Coefficient Table Built",
                                  "L Coefficient Table Built",
                                  "Flow Offset Set",
                                  "Startup Message",
                                  "Transmission Started",
                                  "First Power Up After Firmware Update"};

        public Form1()
        {
            InitializeComponent();
            list_comport();
            //bmp = new Bitmap(804, 212);
            bmp = new Bitmap(804, (200)); //80*2,5
            g = Graphics.FromImage(bmp);
            
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            //comboBox_N2O.SelectedItem = "0%";
            //comboBox_O2.SelectedItem = "20%";

            for (index = 0; index < pictureBox1.Width; index++)
            {
                if (index % 6 == 0)
                {
                    g.DrawRectangle(new Pen(Color.Gray, 1), index, bmp.Height - 50, 1, 1);
                    g.DrawRectangle(new Pen(Color.Gray, 1), index, bmp.Height - 100, 1, 1);
                    g.DrawRectangle(new Pen(Color.Gray, 1), index, bmp.Height - 150, 1, 1);
                }
                if (index % 4 == 0)
                {
                    g.DrawRectangle(new Pen(Color.Gray, 1), index, bmp.Height - 25, 1, 1);
                    g.DrawRectangle(new Pen(Color.Gray, 1), index, bmp.Height - 75, 1, 1);
                    g.DrawRectangle(new Pen(Color.Gray, 1), index, bmp.Height - 125, 1, 1);
                    g.DrawRectangle(new Pen(Color.Gray, 1), index, bmp.Height - 175, 1, 1); 
                }
            }
            pictureBox1.Image = bmp;
        }
        
        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void list_comport()
        {
            // Get a list of serial port names. 
            string[] ports = SerialPort.GetPortNames();



            // Display each port name to the console. 
            foreach (string port in ports)
            {
                //mnuCom.DropDownItems.Add(port, null, new EventHandler(port_Click));
                portToolStripMenuItem.DropDownItems.Add(port, null, new EventHandler(port_Click));
            }
        }

        private void port_Click(object sender, EventArgs e)
        {

            if (serialPort1.IsOpen)
                serialPort1.Close();
            serialPort1.ReadBufferSize = 64;
            serialPort1.ReceivedBytesThreshold = 5;
            string Port = sender.ToString();
            serialPort1.PortName = Port;

            try
            {
                serialPort1.Open();
                toolStripStatusLabel_Port.Text = Port;
                
            }
            catch
            {
                MessageBox.Show("Serial port " + serialPort1.PortName +
                       " cannot be opened!", "RS232 tester",
                       MessageBoxButtons.OK, MessageBoxIcon.Warning);

            }
            if (serialPort1.IsOpen)
            {
                radioButton_50Hz.PerformClick();
                radioButton_100.PerformClick();
                Send_command ("t7");
                comboBox_O2.SelectedIndex = 0;
                comboBox_N2O.SelectedIndex = 0;
            

            }
        }

        Graphics g;
        Bitmap bmp;
        int old_wave;

        private void Send_command(string message)
        {
            if (serialPort1.IsOpen)
                serialPort1.WriteLine(message);
        }
       
        //Int16 Co2pixel;  
        private void Display(byte command,Int16 value)
        {
            
            if (command == 0x00)
            {
                double temp = ((double)(value / 10.0));
                label_etCO2.Text = temp.ToString("0.0");
            }
            else if (command == 0x01)
            {
                double temp = ((double)(value / 10.0));
                label_temp.Text = temp.ToString(".0");
            }
            else if (command == 0x02)
            {
                label_pressure.Text = value.ToString();
            }
            
            else if (command == 0x05)
            {
                //Co2pixel = value;
                //value = (Int16)(value * 2.5 /10);

                if (index < pictureBox1.Width)
                {
                    index = index + 2;
                }
                else
                {
                    index = 0;
                }

                //value =(Int16 ) (bmp.Height - value);

                g.DrawLine(new Pen(Color.White, 2), index, 1, index, bmp.Height);
                g.DrawLine(new Pen(Color.White, 2), index + 1, 1, index + 1, bmp.Height);


                if (index % 6 == 0)
                {
                    g.DrawRectangle(new Pen(Color.Gray, 1), index, bmp.Height - 50, 1, 1);
                    g.DrawRectangle(new Pen(Color.Gray, 1), index, bmp.Height - 100, 1, 1);
                    g.DrawRectangle(new Pen(Color.Gray, 1), index, bmp.Height - 150, 1, 1);
                }
                if (index % 4 == 0)
                {
                    g.DrawRectangle(new Pen(Color.Gray, 1), index, bmp.Height - 25, 1, 1);
                    g.DrawRectangle(new Pen(Color.Gray, 1), index, bmp.Height - 75, 1, 1);
                    g.DrawRectangle(new Pen(Color.Gray, 1), index, bmp.Height - 125, 1, 1);
                    g.DrawRectangle(new Pen(Color.Gray, 1), index, bmp.Height - 175, 1, 1);
                }
                if (index >= 1)
                {
                    g.DrawLine(new Pen(Color.Blue, 2), index - 2, (Int16)(bmp.Height - old_wave* 2.5 / 10 ), index, (Int16)(bmp.Height - value* 2.5 / 10 ));
                    g.DrawLine(new Pen(Color.DarkOrange, 2), index, 200, index, (Int16)(bmp.Height - value * 2.5 / 10));

                }

                old_wave = value;
                if (Outfilectr > 0)
                {
                    {
                        Outfile.Write(this.label_etCO2.Text);
                        Outfile.Write("\t");
                        Outfile.Write(this.label_inCO2.Text);
                        Outfile.Write("\t");
                        Outfile.Write(this.label_Resp.Text);
                        Outfile.Write("\t");
                        Outfile.Write(this.label_temp.Text);
                        Outfile.Write("\t");
                        Outfile.Write(this.label_pressure.Text);
                        Outfile.Write("\t");
                        Outfile.Write(this.label_flow.Text);
                        Outfile.Write("\t");
                        Outfile.Write(this.label_FiO2.Text);
                        Outfile.Write("\t");
                        Outfile.Write(value.ToString());
                        Outfile.Write("\t");
                        Outfile.WriteLine(Outfilectr);
                    }
                    Outfilectr++;
                }
                
            }
            else if (command == 0x06)
            {
                label_Resp.Text = value.ToString();
            }
            else if (command == 0x0C)
            {
                label_flow.Text = value.ToString();
            }
            else if (command == 0x0D)
            {
                label_inCO2.Text = ((double)(value / 10.0)).ToString("0,0");
            }
            else if (command == 0x0E)
            {
                breath_trigger = true;
            }
            else if (command == 0x0F)
            {
                extPanel3.Visible = true;
                label_FiO2.Text = ((double)(value / 10.0)).ToString("0,0");
            }

            
            
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            toolStripStatusLabel_Time.Text = DateTime.Now.ToLongTimeString();
        }

        private void serialPort1_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            string buffer = "";
            byte buf = 0;
            byte[] byte_buffer ={ 0, 0,0 }; 
            while (serialPort1.BytesToRead > 2)
            {
                buf = (byte)(serialPort1.ReadByte());

                if ((buf & 0x80) == 0x80)
                {
                    if (((buf & 0x78) >> 3) == 0x03)
                    {

                        char buff = (char)(serialPort1.ReadByte());
                        
                        while (buff != 0xa0)
                        {
                            buff = (char)(serialPort1.ReadByte());
                            buffer += buff;
                           
                        }
                        int index = buffer.IndexOf('\n');
                         string temp= buffer.Remove(index);
                        this.BeginInvoke(new InfoDeleg(Info), new object[] { temp });


                    }
                    else if (((buf & 0x78) >> 3) == 0x07)
                    {
                        //buf = (byte)(serialPort1.ReadByte());
                        //if ((buf & 0x18) < 20)
                        {
                            char buff = (char)(serialPort1.ReadByte());
                            //buffer = StatusMessage[buf & 0x18];
                            while (buff != 0xc0)
                            {
                                buff = (char)(serialPort1.ReadByte());
                                //if (serialPort1.BytesToRead >= 8)
                                buffer += buff;
                            }
                            int index = buffer.IndexOf('\n');
                            string temp = buffer.Remove(index);

                            this.BeginInvoke(new ErrorDeleg(Error), new object[] { temp });
                        }
                    }
                    else if (((buf & 0x78) >> 3) == 0x0A)
                    {
                        //buf = (byte)(serialPort1.ReadByte());
                        //if ((buf & 0x18) < 20)
                        {
                            char buff = (char)(serialPort1.ReadByte());
                            //buffer = StatusMessage[buf & 0x18];
                            while (((buff ))!= 0xB0)
                            {
                                buff = (char)(serialPort1.ReadByte());
                                //if (serialPort1.BytesToRead >= 8)
                                buffer += buff;
                            }
                           int index = buffer.LastIndexOf(' ');
                            string temp = buffer.Remove((index));

                            this.BeginInvoke(new ParameterDeleg(Parameter), new object[] { temp });
                        }
                    }

                    else
                    {
                        byte_buffer[0] = buf;
                        byte_buffer[1] = (byte)(serialPort1.ReadByte());


                        byte command = (byte)((byte_buffer[0] & 0x78) >> 3);
                        
                        Int16 value = byte_buffer[1];
                        value += (Int16)((byte_buffer[0] & 0x04) << 5);
                        value += (Int16)((byte_buffer[0] & 0x03) << 8);
                        this.BeginInvoke(new displayDeleg(Display), new object[] { command, value });

                    }
                }
            }
        }

        private void Info(string message)
        {
            label_Status.Text = message;
        }

        private void Error(string message)
        {
            timer3.Start();
            label_Error.Text = message;
        }

        private void Parameter(string message)
        {
            MessageBox.Show("Paramter Modul: \n\n" + message 
                       ,"Modulinfo",
                       MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AboutBox1 about = new AboutBox1();
            about.Show();
        }

        StreamWriter Outfile;
        int Outfilectr = 0;

        private void savefileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (savefileToolStripMenuItem.Checked == false)
            {
                SaveFileDialog dialog = new SaveFileDialog();
                dialog.Title = "Select a file";
                dialog.DefaultExt = "txt";
                dialog.Filter =
                "Text files (*.txt)|*.txt|All files (*.*)|*.*";

                if (dialog.ShowDialog() == DialogResult.OK)
                {
                    Outfile = new StreamWriter(dialog.FileName);
                    Outfilectr = 1;
                    savefileToolStripMenuItem.Checked = true;
                    Outfile.WriteLine("EG01200");
                    Outfile.WriteLine("etCO2\tinCO2\tResp\tTemp\tPressur\tFlow\tFiO2\tWave");

                    toolStripStatusLabel1.Text = dialog.FileName;
                }
            } 
            else if (savefileToolStripMenuItem.Checked == true)
            {
                savefileToolStripMenuItem.Checked = false;
                Outfilectr = 0;
                Outfile.Close();
                toolStripStatusLabel1.Text = "No savefile";

            }
        }

        private void set_speed(object sender, EventArgs e)
        {
            if (((RadioButton)sender).Text == "50")
            {
                Send_command("s0");
            }
            else if (((RadioButton)sender).Text == "25")
            {
                Send_command("s1");
            }
               
        }
        
        private void set_SampleRate(object sender, EventArgs e)
        {
            if (((RadioButton)sender).Text == "60")
            {
                Send_command("f0");
            }
            else if (((RadioButton)sender).Text == "100")
            {
                Send_command("f1");
            }
            else if (((RadioButton)sender).Text == "150")
            {
                Send_command("f2");
            }
        }

        private void button_FiO2Cal_Click(object sender, EventArgs e)
        {
            Send_command("i");
        }

        private void button_5_Cal_Click(object sender, EventArgs e)
        {
            Send_command("5");
        }

        private void button_ZeroCal_Click(object sender, EventArgs e)
        {
            Send_command("z");
        }

        private void comboBox_N2O_SelectedIndexChanged(object sender, EventArgs e)
        {
            string temp;
            temp = "n"+(comboBox_N2O.SelectedIndex).ToString();
            Send_command(temp);
        }

        private void comboBox_O2_SelectedIndexChanged(object sender, EventArgs e)
        {
            string temp;
            temp = "o" + ((comboBox_O2.SelectedIndex)+2).ToString();
            Send_command(temp);
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (serialPort1.IsOpen)
                serialPort1.Close();
            if (Outfilectr > 0)
                Outfile.Close();

        }

        private void timer2_Tick(object sender, EventArgs e)
        {
            pictureBox1.Image = bmp;
        }

        private void timer3_Tick(object sender, EventArgs e)
        {
            timer3.Stop();
            label_Error.Text = "";
            
        }

        private void modulinfoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Send_command("p");
        }

        
    }

    class ExtPanel : Panel
    {
        private Color m_colorBorder = Color.Transparent;
        public ExtPanel()
            : base()
        {
            this.SetStyle(ControlStyles.UserPaint, true);
            //this.BackColor = Color.White;
            this.BorderColor = Color.Blue;
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);
            e.Graphics.DrawRectangle(new Pen(new SolidBrush(m_colorBorder),2),
            e.ClipRectangle.Left+1, e.ClipRectangle.Top+1,
            e.ClipRectangle.Width-2 , e.ClipRectangle.Height-2 );
            
        }

        public Color BorderColor
        {
            get { return m_colorBorder; }
            set { m_colorBorder = value; }
        }
    }
}